@@ -10,8 +10,6 @@ from replifactory.culture.culture_functions import inoculate
 from replifactory.growth_rate import calculate_last_growth_rate
 from replifactory.util import write_variable
 
-
-from replifactory.culture.culture_functions import is_active
 from replifactory.util import read_csv_tail
 
 
@@ -46,8 +44,8 @@ class BlankCulture:
         self._log2_dilution_coefficient = 0
         self._inoculation_time = None
         self._samples_collected = {}
-        self._is_aborted = False
-        self._last_dilution_start_time = self.experiment_start_time
+        self._is_active = False
+        self._last_dilution_start_time = None
 
         self._mu_max_measured = 0
         self._t_doubling_min_measured = np.inf
@@ -56,8 +54,13 @@ class BlankCulture:
                                     3: None,
                                     4: None}
 
-    def update(self):
+    def description_text(self):
+        t = f"""
+BLANK culture, measures OD every minute. Pumps deactivated.
+        """
+        return t
 
+    def update(self):
         pass
 
     @property
@@ -138,10 +141,10 @@ class BlankCulture:
         return fig
 
     def is_active(self):
-        if type(self) is BlankCulture:
-            return False
-        else:
-            return is_active(culture=self)
+        # if type(self) is BlankCulture:
+        #     return False
+        # else:
+            return self._is_active
 
     # def show_parameters(self, increase_verbosity=False):
     #     if increase_verbosity:
@@ -331,8 +334,8 @@ class BlankCulture:
         # TODO preserve drug concentrations while sampling
         device = self.device
         vial = self.vial_number
-        device.locks_vials[vial].acquire()
-        device.lock_pumps.acquire()
+        assert device.locks_vials[vial].acquire(timeout=60)
+        assert device.lock_pumps.acquire(timeout=60)
         q = input("ready to collect %d ml sample? [y/n]" % sample_volume)
         if q == "y":
             assert not device.is_pumping(), "pumping in progress"  # must have been checked before releasing the pump_number lock
@@ -357,8 +360,7 @@ class BlankCulture:
             device.valves.close(valve=vial)
         device.locks_vials[vial].release()
         device.lock_pumps.release()
-        self.device.locks_vials[self.vial_number].acquire()
-        self.device.lock_pumps.acquire()
+
 
     def show_parameters(self, increase_verbosity=False):
         if self.is_active():
@@ -389,7 +391,7 @@ class BlankCulture:
                     print("       inoculated: NO")
 
                 print("samples_collected:", self._samples_collected)
-                print("       is_aborted:", self._is_aborted)
+                print("       is_active:", self.is_active())
                 print("         od_blank:", self.od_blank)
                 if hasattr(self, "default_dilution_volume"):
                     vial_volume = self.dead_volume
@@ -404,7 +406,7 @@ class BlankCulture:
         inoculate(culture=self, name=name, description=description)
         self._mu_max_measured = 0
         self._t_doubling_min_measured = np.inf
-        self._is_aborted = False
+        # self._is_active = True
         self.save()
 
     # @property
