@@ -1,9 +1,13 @@
 import time
+import traceback
+from replifactory.util import bcolors
+
 import pyftdi.i2c
 from pyftdi.spi import SpiController
 import threading
 import os
 import numpy as np
+import pandas as pd
 import yaml
 from matplotlib import pyplot as plt
 
@@ -16,9 +20,12 @@ from replifactory.device.pump import Pump, calibrate_pump
 from replifactory.device.thermometers import Thermometers
 from replifactory.device.valves import Valves
 from replifactory.device.dilution import make_dilution
+from replifactory.culture.blank import BlankCulture
+
 from replifactory.util import CultureDict
 from replifactory.loading import save_object, load_config, load_object
 from replifactory.device.workers import QueueWorker
+from replifactory.device.test_hardware import Testing
 
 
 class BaseDevice:
@@ -30,27 +37,21 @@ class BaseDevice:
     PORT_GPIO_MULTIPLEXER_ADC = 0x21  # PCA 9555
     PORT_GPIO_MULTIPLEXER_STIRRERS = 0x25  # PCA 9555
     PORT_THERMOMETER_VIALS = 0x49  # ADT 75  #0x4C?
+    PORT_THERMOMETER_VIALS_v4 = 0x4C  # device version 4
     PORT_THERMOMETER_BOARD = 0x48  # ADT 75
     PORT_PWM = 0x70  # PCA9685
 
     def __init__(self, ftdi_address='ftdi://ftdi:2232h', connect=False, directory=None):
         self.ftdi_address = ftdi_address
         self.directory = directory
-        self.cultures = CultureDict(self)
-        self.cultures[1] = None
-        self.cultures[2] = None
-        self.cultures[3] = None
-        self.cultures[4] = None
-        self.cultures[5] = None
-        self.cultures[6] = None
-        self.cultures[7] = None
+
         self.active_pumps = (1, 4)
 
         self.dilution_worker = None
         self.od_worker = None
 
-        self.hard_stop_trigger = False
-        self.soft_stop_trigger = False
+        # self.hard_stop_trigger = False
+        # self.soft_stop_trigger = False
 
         self.i2c = None
         self.spi = None
@@ -97,6 +98,14 @@ class BaseDevice:
         self.thermometers = Thermometers(device=self)
         # if self.ftdi_address is not None:
         #     self.connect()
+        self.cultures = CultureDict(self)
+        self.cultures[1] = None
+        self.cultures[2] = None
+        self.cultures[3] = None
+        self.cultures[4] = None
+        self.cultures[5] = None
+        self.cultures[6] = None
+        self.cultures[7] = None
         if self.directory is not None:
             try:
                 self.load()
@@ -105,6 +114,17 @@ class BaseDevice:
         if connect:
             self.connect()
 
+        self.testing = Testing(self)
+
+    def make_all_blank_cultures(self):
+        self.cultures[1] = BlankCulture(directory=self.directory, vial_number=1)
+        self.cultures[2] = BlankCulture(directory=self.directory, vial_number=2)
+        self.cultures[3] = BlankCulture(directory=self.directory, vial_number=3)
+        self.cultures[4] = BlankCulture(directory=self.directory, vial_number=4)
+        self.cultures[5] = BlankCulture(directory=self.directory, vial_number=5)
+        self.cultures[6] = BlankCulture(directory=self.directory, vial_number=6)
+        self.cultures[7] = BlankCulture(directory=self.directory, vial_number=7)
+
     def connect(self, ftdi_address='ftdi://ftdi:2232h', fit_calibration=False):
         # if ftdi_address is None:
         #     ftdi_address = self.ftdi_address
@@ -143,6 +163,11 @@ class BaseDevice:
         for lock in self.locks_vials.values():
             if lock.locked():
                 lock.release()
+        try:
+            self.pwm_controller.play_turn_on_sound()
+            self.lasers.blink()
+        except:
+            traceback.print_exc()
 
     def pump_to(self, vial, p1=0, p2=0, p3=0, p4=0):
         if self.valves.not_all_closed():
@@ -208,8 +233,8 @@ class BaseDevice:
             print("Temperature measurement not queued. od thread queue is not empty.")
 
     def prime_tubing(self, vial_number, pump_number, volume):
-        self.lock_pumps.acquire()
-        self.locks_vials[vial_number].acquire()
+        assert self.lock_pumps.acquire(timeout=10)
+        assert self.locks_vials[vial_number].acquire(timeout=10)
         self.valves.open(valve=vial_number)
         for i in range(20):
             q = input("%.2f ml of air available in pump_number %d tubing to vial %d?" % (volume,pump_number,vial_number))
@@ -249,12 +274,12 @@ class BaseDevice:
         return any(p.is_pumping() for p in [self.pump1, self.pump2, self.pump3, self.pump4])
 
     def emergency_stop(self):
-        self.hard_stop_trigger = True
+        # self.hard_stop_trigger = True
         self.pump1.stop()
         self.pump2.stop()
         self.pump3.stop()
         self.pump4.stop()
-        self.pwm_controller.stop_all()
+        self.stirrers.emergency_stop()
 
     def is_connected(self):
         """
@@ -274,8 +299,9 @@ class BaseDevice:
         save_object(self, filepath=config_path)
 
     def load(self):
-        self.file_lock.acquire()
+        assert self.file_lock.acquire(timeout=60)
         config_path = os.path.join(self.directory, "device_config.yaml")
+        print(config_path, "loading")
         load_config(self, filepath=config_path)
         self.file_lock.release()
         # print("Loaded device config.")
@@ -356,73 +382,69 @@ class BaseDevice:
         for v in vials:
             assert 1 <= v <= 7
         for v in vials:
-            self.locks_vials[v].acquire()
-            self.lock_pumps.acquire()
-            self.valves.open(v)
-            self.pump1.pump(volume)
-            while self.is_pumping():
-                time.sleep(0.5)
-            time.sleep(2)
-            assert not self.is_pumping()
-            self.valves.close(v)
-            self.locks_vials[v].release()
-            self.lock_pumps.release()
+            try:
+                assert self.locks_vials[v].acquire(timeout=60)
+                assert self.lock_pumps.acquire(timeout=60)
+                self.valves.open(v)
+                self.pump1.pump(volume)
+                while self.is_pumping():
+                    time.sleep(0.5)
+                time.sleep(2)
+                assert not self.is_pumping()
+                self.valves.close(v)
+            finally:
+                self.locks_vials[v].release()
+                self.lock_pumps.release()
 
     def aspirate_vials(self, volume=24, vials=(1, 2, 3, 4, 5, 6, 7)):
         for v in vials:
             assert 1 <= v <= 7
         for v in vials:
-            self.locks_vials[v].acquire()
-            self.lock_pumps.acquire()
-            self.valves.open(v)
-            self.pump4.pump(volume)
-            while self.is_pumping():
-                time.sleep(0.5)
-            time.sleep(2)
-            assert not self.is_pumping(), "device "
-            self.valves.close(v)
-            self.locks_vials[v].release()
-            self.lock_pumps.release()
+            try:
+                assert self.locks_vials[v].acquire(timeout=60)
+                assert self.lock_pumps.acquire(timeout=60)
+                self.valves.open(v)
+                self.pump4.pump(volume)
+                while self.is_pumping():
+                    time.sleep(0.5)
+                time.sleep(2)
+                assert not self.is_pumping(), "device "
+                self.valves.close(v)
+            finally:
+                self.locks_vials[v].release()
+                self.lock_pumps.release()
 
-    def check_all(self):
-        assert not self.hard_stop_trigger
-        assert not self.soft_stop_trigger
+    def run_self_test(self):
+        # assert not self.hard_stop_trigger
+        # assert not self.soft_stop_trigger
         # if not self.is_connected():
         #     self.connect()
         if not self.is_connected():
-            print("Device not connected.")
-        assert not self.file_lock.locked()
-        assert not self.lock_pumps.locked()
-        for lock in self.locks_vials.values():
-            assert not lock.locked()
-        assert not self.is_pumping()
-        self.check_parameters()
-        self.check_cultures()
+            print(bcolors.FAIL + "Device not connected." + bcolors.ENDC)
+        else:
+            if self.directory is None:
+                print(bcolors.FAIL + "Device directory undefined" + bcolors.ENDC)
+            else:
+                if os.path.exists(self.directory):
+                    print("Device directory: " + bcolors.OKBLUE + os.path.abspath(self.directory) + bcolors.ENDC)
+                else:
+                    print("Device directory doesn't exist: " + bcolors.FAIL + os.path.abspath(self.directory) + bcolors.ENDC)
+
+            assert not self.file_lock.locked()
+            assert not self.lock_pumps.locked()
+            for lock in self.locks_vials.values():
+                assert not lock.locked()
+            assert not self.is_pumping()
+            self.check_parameters()
+            self.check_cultures()
 
     def check_parameters(self):
-        if self.pump1.stock_volume is None:
-            print("Pump1 stock volume unknown.")
-        if self.pump2.stock_volume is None:
-            print("Pump2 stock volume unknown.")
-        if self.pump2.stock_concentration is None:
-            print("Pump2 stock concentration unknown.")
-        if self.pump3.stock_volume is None:
-            print("Pump3 stock volume unknown.")
-        if self.pump4.stock_volume is None:
-            print("Free waste bottle volume unknown.")
-
-        for v in self.cultures.keys():
-            if not callable(self.od_sensors[v].calibration_function):
-                print("Vial %d OD calibration function missing." % v)
-        if not callable(self.pump1.calibration_function):
-            print("Pump 1 calibration function missing")
-        if not callable(self.pump2.calibration_function):
-            print("Pump 2 calibration function missing")
-        # if not callable(self.pump3.calibration_function):
-        #     print("Pump 3 calibration function missing")
-        if not callable(self.pump4.calibration_function):
-            print("Pump 4 calibration function missing")
-        print("Self-test complete")
+        cold, hot = self.thermometers.measure_temperature()
+        print("Temperature Vials:\t%.2f °C\nTemperature Board:\t%.2f °C" % (cold, hot))
+        for pump in [self.pump1, self.pump2, self.pump3, self.pump4]:
+            pump.check()
+        for v in range(1, 8):
+            self.od_sensors[v].check()
 
     def check_cultures(self):
         for v in self.cultures.keys():
